package com.ruoyi.common.core.service.impl;

import com.easy.mongodb.core.toolkit.Wraps;
import com.ruoyi.common.constant.UserConstants;
import com.ruoyi.common.core.domain.entity.SysPost;
import com.ruoyi.common.core.domain.entity.SysUserPost;
import com.ruoyi.common.core.mapper.SysPostMapper;
import com.ruoyi.common.core.mapper.SysUserPostMapper;
import com.ruoyi.common.core.service.ISysPostService;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 岗位信息 服务层处理
 *
 * @author ruoyi
 */
@Service
public class SysPostServiceImpl implements ISysPostService {
    @Autowired
    private SysPostMapper postMapper;

    @Autowired
    private SysUserPostMapper userPostMapper;

    /**
     * 查询岗位信息集合
     *
     * @param post 岗位信息
     * @return 岗位信息集合
     */
    @Override
    public List<SysPost> selectPostList(SysPost post) {
        return this.postMapper.selectList(post);
    }

    /**
     * 查询所有岗位
     *
     * @return 岗位列表
     */
    @Override
    public List<SysPost> selectPostAll() {
        return this.postMapper.selectList(new SysPost());
    }

    /**
     * 通过岗位ID查询岗位信息
     *
     * @param postId 岗位ID
     * @return 角色对象信息
     */
    @Override
    public SysPost selectPostById(String postId) {
        return this.postMapper.selectById(postId);
    }

    /**
     * 根据用户ID获取岗位选择框列表
     *
     * @param userId 用户ID
     * @return 选中岗位ID列表
     */
    @Override
    public List<String> selectPostListByUserId(String userId) {
        return this.userPostMapper.selectList(Wraps.lbQ(SysUserPost.class).eq(SysUserPost::getUserId, userId)).stream().map(e -> e.getPostId().toHexString()).collect(Collectors.toList());
    }

    /**
     * 校验岗位名称是否唯一
     *
     * @param post 岗位信息
     * @return 结果
     */
    @Override
    public String checkPostNameUnique(SysPost post) {
        String postId = StringUtils.isNull(post.getId()) ? "-1L" : post.getId();
        SysPost info = this.postMapper.selectOne(Wraps.lbQ(SysPost.class).eq(SysPost::getPostName, post.getPostName()));
        if (StringUtils.isNotNull(info) && !postId.equalsIgnoreCase(info.getId())) {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    /**
     * 校验岗位编码是否唯一
     *
     * @param post 岗位信息
     * @return 结果
     */
    @Override
    public String checkPostCodeUnique(SysPost post) {
        String postId = StringUtils.isNull(post.getId()) ? "-1L" : post.getId();
        SysPost info = this.postMapper.selectOne(Wraps.lbQ(SysPost.class).eq(SysPost::getPostCode, post.getPostCode()));
        if (StringUtils.isNotNull(info) && !postId.equals(info.getId())) {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    /**
     * 通过岗位ID查询岗位使用数量
     *
     * @param postId 岗位ID
     * @return 结果
     */
    @Override
    public int countUserPostById(String postId) {
        return this.userPostMapper.selectCount(Wraps.lbQ(SysUserPost.class).eq(SysUserPost::getPostId, postId)).intValue();
    }

    /**
     * 删除岗位信息
     *
     * @param postId 岗位ID
     * @return 结果
     */
    @Override
    public int deletePostById(String postId) {
        return this.postMapper.deleteById(postId).intValue();
    }

    /**
     * 批量删除岗位信息
     *
     * @param postIds 需要删除的岗位ID
     * @return 结果
     */
    @Override
    public int deletePostByIds(String[] postIds) {
        for (String postId : postIds) {
            SysPost post = this.selectPostById(postId);
            if (this.countUserPostById(postId) > 0) {
                throw new ServiceException(String.format("%1$s已分配,不能删除", post.getPostName()));
            }
        }
        return this.postMapper.deleteBatchIds(Arrays.asList(postIds)).intValue();
    }

    /**
     * 新增保存岗位信息
     *
     * @param post 岗位信息
     * @return 结果
     */
    @Override
    public int insertPost(SysPost post) {
        return this.postMapper.insert(post).intValue();
    }

    /**
     * 修改保存岗位信息
     *
     * @param post 岗位信息
     * @return 结果
     */
    @Override
    public int updatePost(SysPost post) {
        return this.postMapper.updateById(post).intValue();
    }
}
