package com.ruoyi.common.core.service.impl;

import com.easy.mongodb.core.toolkit.Wraps;
import com.ruoyi.common.annotation.DataScope;
import com.ruoyi.common.constant.UserConstants;
import com.ruoyi.common.core.domain.entity.SysRole;
import com.ruoyi.common.core.domain.entity.SysRoleDept;
import com.ruoyi.common.core.domain.entity.SysRoleMenu;
import com.ruoyi.common.core.domain.entity.SysUserRole;
import com.ruoyi.common.core.mapper.*;
import com.ruoyi.common.core.service.ISysRoleService;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.spring.SpringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 角色 业务层处理
 *
 * @author ruoyi
 */
@Service
public class SysRoleServiceImpl implements ISysRoleService {
    @Autowired
    private SysUserMapper userMapper;
    @Autowired
    private SysRoleMapper roleMapper;

    @Autowired
    private SysRoleMenuMapper roleMenuMapper;

    @Autowired
    private SysUserRoleMapper userRoleMapper;

    @Autowired
    private SysRoleDeptMapper roleDeptMapper;

    /**
     * 根据条件分页查询角色数据
     *
     * @param role 角色信息
     * @return 角色数据集合信息
     */
    @Override
    @DataScope(deptAlias = "d")
    public List<SysRole> selectRoleList(SysRole role) {
        return this.roleMapper.selectList(role);
    }

    /**
     * 根据用户ID查询角色
     *
     * @param userId 用户ID
     * @return 角色列表
     */
    @Override
    public List<SysRole> selectRolesByUserId(String userId) {
        List<SysUserRole> userRoleMids = this.userRoleMapper.selectList(Wraps.lbQ(SysUserRole.class).eq(SysUserRole::getUserId, userId));
        List<SysRole> userRoles = this.roleMapper.selectList(Wraps.lbQ(SysRole.class).in(SysRole::getId, userRoleMids.stream().map(e -> e.getRoleId()).collect(Collectors.toList())));
        List<SysRole> roles = this.selectRoleAll();
        for (SysRole role : roles) {
            for (SysRole userRole : userRoles) {
                if (role.getId().equalsIgnoreCase(userRole.getId())) {
                    role.setFlag(true);
                    break;
                }
            }
        }
        return roles;
    }

    /**
     * 根据用户ID查询权限
     *
     * @param userId 用户ID
     * @return 权限列表
     */
    @Override
    public Set<String> selectRolePermissionByUserId(String userId) {
        List<SysUserRole> userRoleMids = this.userRoleMapper.selectList(Wraps.lbQ(SysUserRole.class).eq(SysUserRole::getUserId, userId));
        List<SysRole> perms = this.roleMapper.selectList(Wraps.lbQ(SysRole.class).in(SysRole::getId, userRoleMids.stream().map(e -> e.getRoleId()).collect(Collectors.toList())));
        Set<String> permsSet = new HashSet<>();
        for (SysRole perm : perms) {
            if (StringUtils.isNotNull(perm)) {
                permsSet.addAll(Arrays.asList(perm.getRoleKey().trim().split(",")));
            }
        }
        return permsSet;
    }

    /**
     * 查询所有角色
     *
     * @return 角色列表
     */
    @Override
    public List<SysRole> selectRoleAll() {
        return SpringUtils.getAopProxy(this).selectRoleList(new SysRole());
    }

    /**
     * 根据用户ID获取角色选择框列表
     *
     * @param userId 用户ID
     * @return 选中角色ID列表
     */
    @Override
    public List<String> selectRoleListByUserId(String userId) {
        List<SysUserRole> userRoleMids = this.userRoleMapper.selectList(Wraps.lbQ(SysUserRole.class).eq(SysUserRole::getUserId, userId));
        return userRoleMids.stream().map(e -> e.getRoleId().toHexString()).collect(Collectors.toList());
    }

    /**
     * 通过角色ID查询角色
     *
     * @param roleId 角色ID
     * @return 角色对象信息
     */
    @Override
    public SysRole selectRoleById(String roleId) {
        return this.roleMapper.selectById(roleId);
    }

    /**
     * 校验角色名称是否唯一
     *
     * @param role 角色信息
     * @return 结果
     */
    @Override
    public String checkRoleNameUnique(SysRole role) {
        String roleId = StringUtils.isNull(role.getId()) ? "-1L" : role.getId();
        SysRole info = this.roleMapper.selectOne(Wraps.lbQ(SysRole.class).eq(SysRole::getRoleName, role.getRoleName()));
        if (StringUtils.isNotNull(info) && !roleId.equals(info.getId())) {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    /**
     * 校验角色权限是否唯一
     *
     * @param role 角色信息
     * @return 结果
     */
    @Override
    public String checkRoleKeyUnique(SysRole role) {
        String roleId = StringUtils.isNull(role.getId()) ? "-1L" : role.getId();
        SysRole info = this.roleMapper.selectOne(Wraps.lbQ(SysRole.class).eq(SysRole::getRoleKey, role.getRoleKey()));
        if (StringUtils.isNotNull(info) && !roleId.equals(info.getId())) {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    /**
     * 校验角色是否允许操作
     *
     * @param role 角色信息
     */
    @Override
    public void checkRoleAllowed(SysRole role) {
        if (StringUtils.isNotNull(role.getId()) && role.isAdmin()) {
            throw new ServiceException("不允许操作超级管理员角色");
        }
    }

    /**
     * 校验角色是否有数据权限
     *
     * @param roleId 角色id
     */
    @Override
    public void checkRoleDataScope(String roleId) {
        if (!SecurityUtils.getLoginUser().getUser().isAdmin()) {
            SysRole role = new SysRole();
            role.setId(roleId);
            List<SysRole> roles = SpringUtils.getAopProxy(this).selectRoleList(role);
            if (StringUtils.isEmpty(roles)) {
                throw new ServiceException("没有权限访问角色数据！");
            }
        }
    }

    /**
     * 通过角色ID查询角色使用数量
     *
     * @param roleId 角色ID
     * @return 结果
     */
    @Override
    public int countUserRoleByRoleId(String roleId) {
        return this.userRoleMapper.selectCount(Wraps.lbQ(SysUserRole.class).eq(SysUserRole::getRoleId, roleId)).intValue();
    }

    /**
     * 新增保存角色信息
     *
     * @param role 角色信息
     * @return 结果
     */
    @Override
    @Transactional
    public int insertRole(SysRole role) {
        // 新增角色信息
        this.roleMapper.insert(role);
        return this.insertRoleMenu(role);
    }

    /**
     * 修改保存角色信息
     *
     * @param role 角色信息
     * @return 结果
     */
    @Override
    @Transactional
    public int updateRole(SysRole role) {
        // 修改角色信息
        this.roleMapper.updateById(role);
        // 删除角色与菜单关联
        this.roleMenuMapper.delete(Wraps.lbQ(SysRoleMenu.class).eq(SysRoleMenu::getRoleId, role.getId()));
        return this.insertRoleMenu(role);
    }

    /**
     * 修改角色状态
     *
     * @param role 角色信息
     * @return 结果
     */
    @Override
    public int updateRoleStatus(SysRole role) {
        return this.roleMapper.updateById(role).intValue();
    }

    /**
     * 修改数据权限信息
     *
     * @param role 角色信息
     * @return 结果
     */
    @Override
    @Transactional
    public int authDataScope(SysRole role) {
        // 修改角色信息
        this.roleMapper.updateById(role);
        // 删除角色与部门关联
        this.roleDeptMapper.delete(Wraps.lbQ(SysRoleDept.class).eq(SysRoleDept::getRoleId, role.getId()));
        // 新增角色和部门信息（数据权限）
        return this.insertRoleDept(role);
    }

    /**
     * 新增角色菜单信息
     *
     * @param role 角色对象
     */
    public int insertRoleMenu(SysRole role) {
        int rows = 1;
        // 新增用户与角色管理
        List<SysRoleMenu> list = new ArrayList<SysRoleMenu>();
        for (String menuId : role.getMenuIds()) {
            SysRoleMenu rm = new SysRoleMenu();
            rm.setRoleId(role.getId());
            rm.setMenuId(menuId);
            list.add(rm);
        }
        if (list.size() > 0) {
            rows = this.roleMenuMapper.insertBatch(list).intValue();
        }
        return rows;
    }

    /**
     * 新增角色部门信息(数据权限)
     *
     * @param role 角色对象
     */
    public int insertRoleDept(SysRole role) {
        int rows = 1;
        // 新增角色与部门（数据权限）管理
        List<SysRoleDept> list = new ArrayList<SysRoleDept>();
        for (String deptId : role.getDeptIds()) {
            SysRoleDept rd = new SysRoleDept();
            rd.setRoleId(role.getId());
            rd.setDeptId(deptId);
            list.add(rd);
        }
        if (list.size() > 0) {
            rows = this.roleDeptMapper.insertBatch(list).intValue();
        }
        return rows;
    }

    /**
     * 通过角色ID删除角色
     *
     * @param roleId 角色ID
     * @return 结果
     */
    @Override
    @Transactional
    public int deleteRoleById(String roleId) {
        // 删除角色与菜单关联
        this.roleMenuMapper.delete(Wraps.lbQ(SysRoleMenu.class).eq(SysRoleMenu::getRoleId, roleId));
        // 删除角色与部门关联
        this.roleDeptMapper.delete(Wraps.lbQ(SysRoleDept.class).eq(SysRoleDept::getRoleId, roleId));
        return this.roleMapper.deleteById(roleId).intValue();
    }

    /**
     * 批量删除角色信息
     *
     * @param roleIds 需要删除的角色ID
     * @return 结果
     */
    @Override
    @Transactional
    public int deleteRoleByIds(String[] roleIds) {
        for (String roleId : roleIds) {
            this.checkRoleAllowed(new SysRole(roleId));
            this.checkRoleDataScope(roleId);
            SysRole role = this.selectRoleById(roleId);
            if (this.countUserRoleByRoleId(roleId) > 0) {
                throw new ServiceException(String.format("%1$s已分配,不能删除", role.getRoleName()));
            }
        }
        // 删除角色与菜单关联
        this.roleMenuMapper.delete(Wraps.lbQ(SysRoleMenu.class).in(SysRoleMenu::getRoleId, roleIds));
        // 删除角色与部门关联
        this.roleDeptMapper.delete(Wraps.lbQ(SysRoleDept.class).in(SysRoleDept::getRoleId, roleIds));
        return this.roleMapper.deleteBatchIds(Arrays.asList(roleIds)).intValue();
    }

    /**
     * 取消授权用户角色
     *
     * @param userRole 用户和角色关联信息
     * @return 结果
     */
    @Override
    public int deleteAuthUser(SysUserRole userRole) {
        return this.userRoleMapper.deleteById(userRole).intValue();
    }

    /**
     * 批量取消授权用户角色
     *
     * @param roleId  角色ID
     * @param userIds 需要取消授权的用户数据ID
     * @return 结果
     */
    @Override
    public int deleteAuthUsers(String roleId, String[] userIds) {
        return this.userRoleMapper.delete(Wraps.lbQ(SysUserRole.class).eq(SysUserRole::getRoleId, roleId).in(SysUserRole::getUserId, userIds)).intValue();
    }

    /**
     * 批量选择授权用户角色
     *
     * @param roleId  角色ID
     * @param userIds 需要授权的用户数据ID
     * @return 结果
     */
    @Override
    public int insertAuthUsers(String roleId, String[] userIds) {
        // 新增用户与角色管理
        List<SysUserRole> list = new ArrayList<SysUserRole>();
        for (String userId : userIds) {
            SysUserRole ur = new SysUserRole();
            ur.setUserId(userId);
            ur.setRoleId(roleId);
            list.add(ur);
        }
        return this.userRoleMapper.insertBatch(list).intValue();
    }
}
